#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include <QtGui/QApplication>
#include <QtGui/QWidget>
#include <QtGui/QDialog>
#include <QtGui/QLabel>
#include <QtGui/QPushButton>
#include <QtGui/QHBoxLayout>
#include <QtGui/QListWidget>
#include <QtGui/QListWidgetItem>
#include <QtGui/QMainWindow>
#include <QtGui/QKeyEvent>
#include <QtGui/qdesktopwidget.h>

#define STB_EXAMPLE_URLS "/config/stb_example_urls.txt"
#define WIDTH   1200
#define HEIGHT  700

#include <dune_stb_api.h>

typedef enum dune_stb_video_mode_e
{
    DUNE_STB_VM_NTSC = 0,
    DUNE_STB_VM_480p,
    DUNE_STB_VM_PAL,
    DUNE_STB_VM_576p,
    DUNE_STB_VM_720p50,
    DUNE_STB_VM_720p60,
    DUNE_STB_VM_1080i50,
    DUNE_STB_VM_1080i60,
    DUNE_STB_VM_1080p23,
    DUNE_STB_VM_1080p24,
    DUNE_STB_VM_1080p50,
    DUNE_STB_VM_1080p60
} dune_stb_video_mode_t;

static const char *dune_stb_video_mode_to_str(dune_stb_video_mode_t mode)
{
    switch (mode)
    {
        case DUNE_STB_VM_NTSC:
            return "NTSC";
        case DUNE_STB_VM_480p:
            return "480p";
        case DUNE_STB_VM_PAL:
            return "PAL";
        case DUNE_STB_VM_576p:
            return "576p";
        case DUNE_STB_VM_720p50:
            return "720p50";
        case DUNE_STB_VM_720p60:
            return "720p60";
        case DUNE_STB_VM_1080i50:
            return "1080i50";
        case DUNE_STB_VM_1080i60:
            return "1080i60";
        case DUNE_STB_VM_1080p23:
            return "1080p23";
        case DUNE_STB_VM_1080p24:
            return "1080p24";
        case DUNE_STB_VM_1080p50:
            return "1080p50";
        case DUNE_STB_VM_1080p60:
            return "1080p60";
    }
    return "Unknown";
}

static const char *dune_stb_aspect_ratio_to_str(int asp)
{
    switch (asp)
    {
        case DUNESTBAPI_ASPECT_RATIO_AUTO: return "Auto";
        case DUNESTBAPI_ASPECT_RATIO_16_9: return "16x9";
        case DUNESTBAPI_ASPECT_RATIO_4_3: return "4x3";
    }
    return "Unknown";
}

static const char *dune_stb_vmiptv_state_to_str(
    dunestbapi_vmIptvState state)
{
    switch (state)
    {
        case DUNESTBAPI_VMIPTV_STATE_NOT_INITIALIZED:
            return "NOT_INITIALIZED";
        case DUNESTBAPI_VMIPTV_STATE_INITIALIZING:
            return "INITIALIZING";
        case DUNESTBAPI_VMIPTV_STATE_INITIALIZED:
            return "INITIALIZED";
        case DUNESTBAPI_VMIPTV_STATE_DEINITIALIZING:
            return "DEINITIALIZING";
        default:
            return "Unknown";
    }
}

static const char *dune_stb_vmiptv_event_to_str(
    dunestbapi_vmIptvEvent event)
{
    switch (event)
    {
        case DUNESTBAPI_VMIPTV_EVENT_NO_EVENT:
            return "NO_EVENT";
        case DUNESTBAPI_VMIPTV_EVENT_ERROR_INVALID_CONFIG:
            return "INVALID_CONFIG";
        case DUNESTBAPI_VMIPTV_EVENT_ERROR_CONNECTION_FAILED:
            return "CONNECTION_FAILED";
        case DUNESTBAPI_VMIPTV_EVENT_ERROR_NO_PERMISSION:
            return "ERROR_NO_PERMISSION";
        case DUNESTBAPI_VMIPTV_EVENT_ERROR_OTHER:
            return "ERROR_OTHER";
        default:
            return "Unknown";
    }
}

static const char *dune_stb_vmweb_state_to_str(
    dunestbapi_vmWebState state)
{
    switch (state)
    {
        case DUNESTBAPI_VMWEB_STATE_NOT_INITIALIZED:
            return "NOT_INITIALIZED";
        case DUNESTBAPI_VMWEB_STATE_INITIALIZING:
            return "INITIALIZING";
        case DUNESTBAPI_VMWEB_STATE_INITIALIZED:
            return "INITIALIZED";
        case DUNESTBAPI_VMWEB_STATE_DEINITIALIZING:
            return "DEINITIALIZING";
        default:
            return "Unknown";
    }
}

static const char *dune_stb_vmweb_event_to_str(
    dunestbapi_vmWebEvent event)
{
    switch (event)
    {
        case DUNESTBAPI_VMWEB_EVENT_NO_EVENT:
            return "NO_EVENT";
        case DUNESTBAPI_VMWEB_EVENT_ERROR_INVALID_CONFIG:
            return "INVALID_CONFIG";
        case DUNESTBAPI_VMWEB_EVENT_ERROR_CONNECTION_FAILED:
            return "CONNECTION_FAILED";
        case DUNESTBAPI_VMWEB_EVENT_ERROR_NO_PERMISSION:
            return "ERROR_NO_PERMISSION";
        case DUNESTBAPI_VMWEB_EVENT_ERROR_OTHER:
            return "ERROR_OTHER";
        default:
            return "Unknown";
    }
}

const char *dune_stb_zoom_to_str(int zoom)
{
    switch (zoom)
    {
      case DUNESTBAPI_VIDEO_ZOOM_NORMAL:
        return "none";
    case DUNESTBAPI_VIDEO_ZOOM_ENLARGE:
        return "235_fill_169";
    case DUNESTBAPI_VIDEO_ZOOM_MAKE_WIDER:
        return "43_to_169";
    case DUNESTBAPI_VIDEO_ZOOM_NON_LINEAR_STRETCH:
        return "43_to_169_nonlinear";
    case DUNESTBAPI_VIDEO_ZOOM_NON_LINEAR_STRETCH_TO_FULL_SCREEN:
        return "2x_nonlinear";
    case DUNESTBAPI_VIDEO_ZOOM_MAKE_TALLER:
        return "169_to_43";
    case DUNESTBAPI_VIDEO_ZOOM_CUT_EDGES:
        return "cut_edges";
    case 7:
        return "custom";
    case DUNESTBAPI_VIDEO_ZOOM_FULL_SCREEN:
        return "full_enlarge";
    case DUNESTBAPI_VIDEO_ZOOM_STRETCH_TO_FULL_SCREEN:
        return "full_stretch";
    default:
        return "unknown";
    }
}

static volatile int dune_stb_vmiptv_init_status = 0;

static void dune_stb_vmiptv_event_callback(
    dunestbapi_vmIptvState in_previousState,
    dunestbapi_vmIptvState in_currentState,
    dunestbapi_vmIptvEvent in_event,
    int in_vmiptv_error_code)
{
    fprintf(stderr,
        "VMIptvEvent: %s -> %s (%s, %d)\n",
        dune_stb_vmiptv_state_to_str(in_previousState),
        dune_stb_vmiptv_state_to_str(in_currentState),
        dune_stb_vmiptv_event_to_str(in_event),
        in_vmiptv_error_code);

    if (in_previousState == DUNESTBAPI_VMIPTV_STATE_INITIALIZING)
    {
        if (in_currentState == DUNESTBAPI_VMIPTV_STATE_INITIALIZED)
            dune_stb_vmiptv_init_status = 1;
        else if (in_currentState == DUNESTBAPI_VMIPTV_STATE_NOT_INITIALIZED)
            dune_stb_vmiptv_init_status = -1;
    }
}

static volatile int dune_stb_vmweb_init_status = 0;

static void dune_stb_vmweb_event_callback(
    dunestbapi_vmWebState in_previousState,
    dunestbapi_vmWebState in_currentState,
    dunestbapi_vmWebEvent in_event,
    int in_vmweb_error_code)
{
    fprintf(stderr,
        "VMWebEvent: %s -> %s (%s, %d)\n",
        dune_stb_vmweb_state_to_str(in_previousState),
        dune_stb_vmweb_state_to_str(in_currentState),
        dune_stb_vmweb_event_to_str(in_event),
        in_vmweb_error_code);

    if (in_previousState == DUNESTBAPI_VMWEB_STATE_INITIALIZING)
    {
        if (in_currentState == DUNESTBAPI_VMWEB_STATE_INITIALIZED)
            dune_stb_vmweb_init_status = 1;
        else if (in_currentState == DUNESTBAPI_VMWEB_STATE_NOT_INITIALIZED)
            dune_stb_vmweb_init_status = -1;
    }
}

class VMListWidgetItem : public QListWidgetItem
{
public:
    VMListWidgetItem(const char *caption, int v) :
        QListWidgetItem(caption), _v(v)
    {
    }
    int _v;
};

class MyVideoModeDialog : public QDialog
{
public:
    MyVideoModeDialog(QWidget *p) : QDialog(0, Qt::SplashScreen),
        _p(p), _mode_list(0),
        _aspect_list(0), _power_off(false)
    {
        setAttribute(Qt::WA_QuitOnClose, true);
        setWindowIcon(QIcon());

        _mode_list = new QListWidget();
        _mode_list->setFixedSize(150, 480);

        for (int i = 0; i <= DUNE_STB_VM_1080p60; ++i)
        {
            _mode_list->addItem(
                new VMListWidgetItem(
                    dune_stb_video_mode_to_str((dune_stb_video_mode_t)i), i));
        }

        _aspect_list = new QListWidget();
        {
            _aspect_list->addItem(
                new VMListWidgetItem(dune_stb_aspect_ratio_to_str(0), 0));
            _aspect_list->addItem(
                new VMListWidgetItem(dune_stb_aspect_ratio_to_str(1), 1));
            _aspect_list->addItem(
                new VMListWidgetItem(dune_stb_aspect_ratio_to_str(2), 2));
        }

        setStyleSheet("* { color:rgb(10,10,10,ff); background-color:rgb(ff,ff,ff,ff); padding: 10px ; font-size: 28pt ; border: 1px solid #8f8f91; } windowTitle {font-size: 28pt ; } ");
//        _mode_list->setStyleSheet("* { background-color:rgb(230,230,230); } ");
//        windowTitle.setStyleSheet("* { font-size: 28pt ; }");

        QGridLayout *l1 = new QGridLayout;
        _label = new QLabel();

        l1->addWidget(_label, 0, 0);
        QGridLayout *layout = new QGridLayout;
        layout->addWidget(_mode_list,   0, 0);
        layout->addWidget(_aspect_list, 0, 1);

        l1->addLayout(layout, 1, 0);

        setLayout(l1);
        update();
        show();
    }

    ~MyVideoModeDialog()
    {
    }

    void changed();
    void keyPressEvent(QKeyEvent *event);

    void update()
    {
        int mode = DUNESTBAPI_VIDEO_MODE_NTSC;
        dunestbapi_getVideoMode(&mode);
        if (mode < DUNESTBAPI_VIDEO_MODE_NTSC ||
            mode > DUNESTBAPI_VIDEO_MODE_1080P60)
            mode = DUNESTBAPI_VIDEO_MODE_NTSC;

        _mode_list->setCurrentItem(_mode_list->item(mode));
        _mode = (dune_stb_video_mode_t)mode;

        int aspect = -1;
        dunestbapi_getAspectRatio(&aspect);
        if (aspect < DUNESTBAPI_ASPECT_RATIO_AUTO ||
            aspect > DUNESTBAPI_ASPECT_RATIO_4_3)
            aspect = DUNESTBAPI_ASPECT_RATIO_AUTO;

        _aspect_list->setCurrentItem(_aspect_list->item(aspect));
        _aspect = aspect;

        int zoom = -1;
        dunestbapi_getVideoZoom(&zoom);

        char buffer[1024];
        sprintf(buffer, "Video mode: %-8s %-4s Zoom: %-20s",
            dune_stb_video_mode_to_str(_mode),
                dune_stb_aspect_ratio_to_str(_aspect),
            dune_stb_zoom_to_str(zoom));
        _label->setText(buffer);
    }

    bool power_off() const
    {
        return _power_off;
    }

protected:

    QWidget *_p;

    dune_stb_video_mode_t _mode;
    int                   _aspect;

    QLabel *_label;
    QListWidget *_mode_list;
    QListWidget *_aspect_list;

    bool _power_off;
};

void MyVideoModeDialog::keyPressEvent(QKeyEvent *event)
{
    switch (event->key()) {
        case Qt::Key_Zoom:
        {
            int zoom = -1;
            dunestbapi_getVideoZoom(&zoom);
            if (zoom != -1)
            {
                ++zoom;
                if (zoom == 7)
                    ++zoom;
                if (zoom > DUNESTBAPI_VIDEO_ZOOM_STRETCH_TO_FULL_SCREEN)
                    zoom = DUNESTBAPI_VIDEO_ZOOM_NORMAL;
                dunestbapi_result res = dunestbapi_setVideoZoom(zoom);
                if (res != DUNESTBAPI_OK)
                    fprintf(stderr, "Erorr setting zoom: %d\n", (int)res);

                // TODO: setting zoom is async: add handing via event
                usleep(100000);
                update();
            }
        }
        break;
        case Qt::Key_Return:
        {
            bool changed = false;
            VMListWidgetItem *item;
            item = (VMListWidgetItem *)_mode_list->currentItem();
            if (item)
            {
                if (item->_v != _mode)
                {
                    dunestbapi_setVideoMode(item->_v);
                    changed = true;
                }
            }

            item = (VMListWidgetItem *)_aspect_list->currentItem();
            if (item)
            {
                if (item->_v != _aspect)
                {
                    dunestbapi_setAspectRatio(item->_v);
                    changed = true;
                }
            }

            if (changed)
                update();
        }
        break;
        case Qt::Key_MediaNext:
        case Qt::Key_Right:
        {
            QKeyEvent *ev = new QKeyEvent(QEvent::KeyPress, Qt::Key_Tab, Qt::NoModifier);
            QApplication::postEvent(this, ev);
            event->accept();
        }
        break;
        case Qt::Key_MediaPrevious:
        case Qt::Key_Left:
        {
            QKeyEvent *ev = new QKeyEvent(QEvent::KeyPress, Qt::Key_Tab, Qt::ShiftModifier);
            QApplication::postEvent(this, ev);
            event->accept();
        }
        break;
        case Qt::Key_Standby:
            _power_off = true;
        case Qt::Key_Menu:
        case Qt::Key_F29:
        case Qt::Key_MediaStop:
        case Qt::Key_Backspace:
        {
            close();
        }
        break;
        default:
            QDialog::keyPressEvent(event);
            break;
    }
}

bool launch_dunestb_ui = false;
bool launch_firmware_upgrade = false;

class MyMainWindow : public QMainWindow
{
public:
    MyMainWindow() :
        QMainWindow(0, Qt::SplashScreen),
        _playing(false), _paused(false)
    {
        setAttribute(Qt::WA_QuitOnClose, true);

        setAutoFillBackground(true);

        _list = new QListWidget();

        load_list();

        QRect rect = QApplication::desktop()->screenGeometry();
        _list->setFixedSize(WIDTH, HEIGHT);

        setCentralWidget(_list);
        setStyleSheet("* { color:rgb(10,10,10,ff); background-color:rgb(0,0,0,0); padding: 10px ; font-size: 28pt ; }");
        _list->setStyleSheet("* { background-color:rgb(230,230,230); } ");
        setGeometry((rect.width() - WIDTH)/2, (rect.height() - HEIGHT)/2,
            WIDTH, HEIGHT);

        startTimer(500);
    }

    void load_list(void)
    {
        const char *file_name = STB_EXAMPLE_URLS;
        FILE *fp = fopen(file_name, "r");
        if (fp)
        {
            char buffer[PATH_MAX + 1];
            char *s, *url = NULL;
            QListWidgetItem *first = NULL;
            QListWidgetItem *item = NULL;
            do
            {
                buffer[PATH_MAX] = 0;
                url = fgets(buffer, sizeof(buffer), fp);
                if (!url)
                    break;
                if (url[PATH_MAX] != 0)
                {
                    fprintf(stderr, "Too long URL\n");
                    continue;
                }
                if (url[0] == 0)
                    continue;
                s = strchr(url, '\n');
                if (s)
                    *s = 0;
                item = new QListWidgetItem((const char *)url);
                _list->addItem(item);
                if (!first)
                    first = item;
            }while(1);
            fclose(fp);
            _list->setCurrentItem(first);
        }
        else
        {
            fprintf(stderr, "File: %s not found\n", file_name);
        }
    }

    void stop_playback()
    {
        dunestbapi_stop();
        _playing = false;
        _list->show();
    }

    void start_playback(const char *url)
    {
        fprintf(stderr, "Start playing URL: '%s'\n", url);
        dunestbapi_play(url);
        _list->hide();
        _playing = true;
    }

    void timerEvent(QTimerEvent *event)
    {
        if (_playing)
        {
            dunestbapi_playbackState st;
            dunestbapi_getPlaybackState(&st);
            if (st < DUNESTBAPI_PLAYBACK_STATE_INITIALIZING ||
                st > DUNESTBAPI_PLAYBACK_STATE_BUFFERING)
                stop_playback();
        }
    }

    void keyPressEvent(QKeyEvent *event)
    {
        switch (event->key()) {
            case Qt::Key_F29:
                fprintf(stderr, "Key_Mode_switch\n");
                {
                    MyVideoModeDialog d(this);
                    d.exec();
                    if (d.power_off())
                        QApplication::postEvent(this,
                            new QKeyEvent(QEvent::KeyPress, Qt::Key_Standby, Qt::NoModifier));
                }
                break;
            case Qt::Key_MediaStop:
                fprintf(stderr, "Stop/Standby\n");
                if (!_playing)
                {
                    fprintf(stderr, "Close main window\n");
                    close();
                }
                else
                {
                    fprintf(stderr, "Stop/Standby in playback\n");
                    dunestbapi_stop();
                    _playing = false;
                    _list->show();
                }
                break;
            case Qt::Key_Down:
            case Qt::Key_Up:
                {
                    if (_playing)
                    {
                        fprintf(stderr, "Down/Up pressed in playback\n");
                        QListWidgetItem *item = _list->currentItem();
                        if (!item)
                            break;
                        int row = _list->currentRow();
                        int size = _list->count();
                        if (size == 0)
                            break;

                        if (event->key() == Qt::Key_Up)
                            --row;
                        else
                            ++row;
                        if (row < 0)
                            row = size - 1;
                        if (row >= size)
                            row = 0;

                        _list->setCurrentRow(row);
                        QListWidgetItem *new_item = _list->currentItem();
                        if (new_item && new_item != item)
                        {
                            item = new_item;
                            start_playback(item->text().toStdString().c_str());
                        }
                    }
                    else
                        QMainWindow::keyPressEvent(event);
                }
                break;
            case Qt::Key_Return:
            case Qt::Key_MediaPlay:
                {
                    fprintf(stderr, "Enter or Play\n");
                    if (_playing)
                    {
                        dunestbapi_playbackState st;
                        dunestbapi_getPlaybackState(&st);
                        if (st == DUNESTBAPI_PLAYBACK_STATE_PAUSED)
                        {
                            fprintf(stderr, "Resume\n");
                            dunestbapi_resume();
                        }
                        break;
                    }

                    QListWidgetItem *item = _list->currentItem();
                    if (!item)
                        break;

                    start_playback(item->text().toStdString().c_str());
                }
                break;
            case Qt::Key_Pause:
                {
                    fprintf(stderr, "Pause\n");
                    dunestbapi_playbackState st;
                    dunestbapi_getPlaybackState(&st);

                    if (st == DUNESTBAPI_PLAYBACK_STATE_PLAYING ||
                        st == DUNESTBAPI_PLAYBACK_STATE_SEEKING ||
                        st == DUNESTBAPI_PLAYBACK_STATE_BUFFERING)
                    {
                        fprintf(stderr, "Pause\n");
                        dunestbapi_pause();
                    }
                    else if (st == DUNESTBAPI_PLAYBACK_STATE_PAUSED)
                    {
                        fprintf(stderr, "Resume\n");
                        dunestbapi_resume();
                    }
                    else
                    {
                        fprintf(stderr, "State %d\n", (int)st);
                    }
                }
                break;
            case Qt::Key_MediaRecord:
                launch_firmware_upgrade = true;
            case Qt::Key_Standby:
                dunestbapi_setExitMode(DUNESTBAPI_EXIT_MODE_POWER_OFF);
            case Qt::Key_LaunchA:
            case Qt::Key_Menu:
            case Qt::Key_F27:
                {
                    fprintf(stderr, "Switch to Dune native UI\n");
                    if (_playing)
                    {
                        dunestbapi_stop();
                        _playing = false;
                    }
                    // not clear how to release as much as need
                    // without closing windows
                    if (event->key() != Qt::Key_Standby)
                        launch_dunestb_ui = true;
                    close();
                }
                break;
            default:
                QMainWindow::keyPressEvent(event);
                break;
        }
    }

protected:
    QListWidget *_list;
    bool _playing;
    bool _paused;
};

static void dune_configure_and_init_verimatrix()
{
    bool enable_vmiptv = false;
    bool start_vmiptv_init_before_playback = false;
    bool wait_vmiptv_init_before_playback = false;

    bool enable_vmweb = false;
    bool start_vmweb_init_before_playback = false;
    bool wait_vmweb_init_before_playback = false;

    if (enable_vmiptv)
    {
        dunestbapi_setVMIptvEventCallback(dune_stb_vmiptv_event_callback);

        dunestbapi_setVMIptvCompanyName("<todo>");
        dunestbapi_setVMIptvServerAddress("<todo>");
        dunestbapi_setVMIptvPreferredVKS("<todo>");
        dunestbapi_setVMIptvTimeout(1);
        dunestbapi_setVMIptvErrorLevel(5);
        dunestbapi_setVMIptvMinKeyRetryInterval(300);

        dunestbapi_configureVMIptv();

        // NOTE: this is optional. Verimatrix IPTV initialization is
        // performed automatically when IPTV playback is started.
        if (start_vmiptv_init_before_playback)
            dunestbapi_startVMIptvInitialization();
    }

    if (enable_vmweb)
    {
        dunestbapi_setVMWebEventCallback(dune_stb_vmweb_event_callback);

        dunestbapi_setVMWebBootAddress("<todo>");

        dunestbapi_configureVMWeb();

        // NOTE: this is optional. Verimatrix Web initialization is
        // performed automatically when HLS playback is started.
        if (start_vmweb_init_before_playback)
            dunestbapi_startVMWebInitialization();
    }

    // NOTE: this is optional. Playback won't start until Verimatrix IPTV
    // initialization is finished.
    if (enable_vmiptv &&
        start_vmiptv_init_before_playback &&
        wait_vmiptv_init_before_playback)
    {
        fprintf(stderr, "Waiting for Verimatrix IPTV initialization...\n");
        while (dune_stb_vmiptv_init_status == 0)
        {
            fprintf(stderr, "... still waiting\n");
            sleep(1);
        }

        fprintf(stderr,
            "Verimatrix IPTV initialization %s\n",
            (dune_stb_vmiptv_init_status > 0) ? "succeeded" : "failed");
    }

    // NOTE: this is optional. Playback won't start until Verimatrix Web
    // initialization is finished.
    if (enable_vmweb &&
        start_vmweb_init_before_playback &&
        wait_vmweb_init_before_playback)
    {
        fprintf(stderr, "Waiting for Verimatrix Web initialization...\n");
        while (dune_stb_vmweb_init_status == 0)
        {
            fprintf(stderr, "... still waiting\n");
            sleep(1);
        }

        fprintf(stderr,
            "Verimatrix Web initialization %s\n",
            (dune_stb_vmweb_init_status > 0) ? "succeeded" : "failed");
    }
}

static void dune_info(void)
{
    fprintf(stderr, "\n\n================================================\n\n");
    char *str;
    dunestbapi_getApiVersion(&str);
    fprintf(stderr, "    dunestbapi_getApiVersion:   %s\n", str);
    free(str);

    dunestbapi_getProductId(&str);
    fprintf(stderr, "    dunestbapi_getProductId:    %s\n", str);
    free(str);

    dunestbapi_getSerialNumber(&str);
    fprintf(stderr, "    dunestbapi_getSerialNumber: %s\n", str);
    free(str);

    dunestbapi_getFirmwareVersion(&str);
    fprintf(stderr, "    getFirmwareVersion: %s\n", str);
    free(str);
    fprintf(stderr, "\n\n================================================\n\n");
}

int run(int argc, char *argv[])
{
    QApplication a(argc, argv);
    while (true)
    {
        launch_dunestb_ui = false;
        launch_firmware_upgrade = false;

        MyMainWindow w;
        w.show();
        a.exec();

        if (launch_firmware_upgrade)
        {
            char *url = 0;
            char *version = 0;

            if (dunestbapi_getFirmwareUpgradeUrl(&url) == DUNESTBAPI_OK &&
                dunestbapi_getFirmwareUpgradeVersion(&version) == DUNESTBAPI_OK)
            {
                dunestbapi_startFirmwareUpgrade(url, version);
            }

            if (url)
                free(url);
            if (version)
                free(version);
        }

        //TODO check power off from dunestbapi_nativeUI
        if (!launch_dunestb_ui)
            break;

        if (dunestbapi_nativeUi() != DUNESTBAPI_OK)
            break;
    }
    return 0;
}

int main(int argc, char **argv)
{
    if (dunestbapi_init(&argc, argv) != DUNESTBAPI_OK)
    {
        fprintf(stderr, "Cannot initialize Dune STB API\n");
        return 1;
    }

    dune_info();
    dunestbapi_setScreenSize(DUNESTBAPI_SCREEN_SIZE_1920_1080);

    dune_configure_and_init_verimatrix();

    int qargc = 2;
    char *qargv[2];
    qargv[0] = strdup("./stb");
    qargv[1] = strdup("-qws");

    int res = run(qargc, qargv);

    dunestbapi_deinit();

    return res;
}
