#ifndef __HD_NATIVE_CODE_PLUGIN_H__
#define __HD_NATIVE_CODE_PLUGIN_H__

#include <stdio.h>

/*************************************************************************/

typedef struct dune_stb_core_s
{
    int size;
    int version_major;
    int version_minor;
    int reserved;
} dune_stb_core_t;

/*************************************************************************/

// native code plugin library name
#define NATIVE_PLUGIN_LIB_NAME "native_code.so"

// native code plugin entry point function type ("C" notation)
// returns NULL on success or error description on failure 
// allocated in dymanic memory via malloc
// Before return plugin must stop all started threads and release 
// any used resources
typedef char *(*native_code_plugin_run_func_t)(dune_stb_core_t *in, 
    const char *arg);

// native code plugin entry point function name
#define NATIVE_CODE_PLUGIN_RUN_FUNC       dune_run_native_code
#define NATIVE_CODE_PLUGIN_RUN_FUNC_NAME "dune_run_native_code"

/*************************************************************************/

static inline int native_code_plugin_stb_version_magor(void)
{
    return 
#include "native_code_plugin_stb_version_major.inc"
;
}

/*************************************************************************/

static inline int native_code_plugin_stb_version_minor(void)
{
    return 
#include "native_code_plugin_stb_version_minor.inc"
;
}

/*************************************************************************/

typedef struct dune_stb_api_s
{
    dune_stb_core_t core;

    //plugin can redirects its log to a different file
    void (*log_redirect_to_file)(const char *file, int do_append);

    //must be call periodically by plugin to perform log rotation
    //once or twice calls per second is recomended
    void (*log_rotate_if_needed)(void);

    //common reaction on eject button
    void (*eject_media)(void);

    //common reaction on power off button
    void (*request_power_off)(void);

    //plugins intended to use directfb must call this method
    //before any directfb usage.
    //returns 0 on success and non zero error code on error
    //or directfb mode not allowed
    int  (*enable_directfb_mode)(void);

    // TODO: stb gfx access

    // stb video player access
    // plugin can use dunestbapi_xxx methods if presented
    // for native playback access. see dune_stb_api.h for details

    // TODO: settings access
    
    // may be plugin APPSTORE or PERSISTFS like access
    // shell browser access
} dune_stb_api_t;

/*************************************************************************/
// method helper to check supported stb interface
static inline dune_stb_api_t *native_code_plugin_cast_stb(
    dune_stb_core_t *in)
{
    if (!in)
        return NULL;

    if (in->version_major != native_code_plugin_stb_version_magor())
        return NULL;

    if (in->version_minor < native_code_plugin_stb_version_minor())
        return NULL;

    if (in->size < (int)sizeof(dune_stb_api_t))
        return NULL;
    
    return (dune_stb_api_t *)in;
}

/*************************************************************************/

#endif //__HD_NATIVE_CODE_PLUGIN_H__
